package com.loafle.overflow.crawler.mongo;

import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.crawler.config.Item;
import com.loafle.overflow.crawler.config.Query;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.ServerAddress;
import com.mongodb.client.MongoDatabase;
import org.bson.Document;

import java.util.HashMap;
import java.util.List;
import java.util.Map;



public class MongoCrawler extends Crawler {

    public Map<String, Object> collectMetric(Config c) throws Exception {

        MongoClient mongoClient = null;
        Map<String, Object> returnMap = new HashMap<String, Object>();

        String dataBaseName = "admin";
        String statusCommand = "serverStatus";

        String targetIP = c.getTarget().getConnection().getIp();
        short targetPort = Short.parseShort(c.getTarget().getConnection().getPort());

        try {

            mongoClient = new MongoClient(new ServerAddress(targetIP, targetPort),
                    MongoClientOptions.builder()
                            .serverSelectionTimeout(1000)
                            .connectTimeout(1000)
                            .socketTimeout(1000).build()
            );

            MongoDatabase database = mongoClient.getDatabase(dataBaseName);
            Document serverStatus = database.runCommand(new Document(statusCommand, 1));

            Map<String,Object> resultMap = new HashMap<String, Object>();
            for (Map.Entry<String, Object> set : serverStatus.entrySet()){
                resultMap.put(set.getKey(), set.getValue());
            }

            List<Item> items = c.getItems();
            for (Item item : items) {
                List<String> metrics = item.getMetrics();
                for (Query query : item.getQueries()) {
                    Map<String, String> re = (Map<String, String>) resultMap.get(query.getQuery());
                    for (int index =0 ; index < query.getKeys().size() ; ++index) {
                        returnMap.put(metrics.get(index),re.get(query.getKeys().get(index)));
                    }
                }
            }
        }catch (Exception e) {
            throw e;
        }finally {
            if (mongoClient != null) {
                mongoClient.close();
            }
        }
        return returnMap;
    }

    @Override
    public Object getInternal(Config config) throws Exception {
        return this.collectMetric(config);
    }
}